#!/usr/bin/env python
# file test_first.py
# author Florent Guiotte <florent.guiotte@irisa.fr>
# version 0.0
# date 12 nov. 2019

def test_first():
    assert True, 'If it fails there is nothing we can do'
